<?php

namespace Drupal\ldap_sso\Form;

use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Url;
use Drupal\ldap_authentication\Helper\LdapAuthenticationConfiguration;
use Drupal\ldap_servers\Entity\Server;

/**
 *
 */
class LdapSsoAdminForm extends ConfigFormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'ldap_sso_admin_form';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['ldap_authentication.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('ldap_authentication.settings');


    $form['sso'] = [
      '#type' => 'fieldset',
      '#title' => t('Single Sign-On'),
    ];

    $form['sso']['enabled'] = [
      '#type' => 'markup',
      '#markup' => '<strong>' . t('Single Sign on is enabled.') .
        '</strong> ' . t('To disable it, disable the LDAP SSO Module on the') . ' ' .
        \Drupal::l(t('Modules Form'), Url::fromRoute('system.modules_list')) .
        '.<p>' . t('Single Sign-On enables ' .
          'users of this site to be authenticated by visiting the URL ' .
          '"user/login/sso, or automatically if selecting "automated ' .
          'single sign-on" below. Set up of LDAP authentication must be ' .
          'performed on the web server. Please review the readme file ' .
          'for more information.') . '</p>',
    ];


    $form['sso']['ssoRemoteUserStripDomainName'] = [
      '#type' => 'checkbox',
      '#title' => t('Strip REMOTE_USER domain name'),
      '#description' => $this->t('Useful when the ' .
        'WWW server provides authentication in the form of user@realm and you ' .
        'want to have both SSO and regular forms based authentication ' .
        'available. Otherwise duplicate accounts with conflicting e-mail ' .
        'addresses may be created.'),
      '#default_value' => $config->get('ldap_authentication_conf.ssoRemoteUserStripDomainName'),
    ];

    $form['sso']['seamlessLogin'] = [
      '#type' => 'checkbox',
      '#title' => t('Turn on automated single sign-on'),
      '#description' => $this->t('This requires that you ' .
        'have operational NTLM or Kerberos authentication turned on for at least ' .
        'the path user/login/sso, or for the whole domain.'),
      '#default_value' => $config->get('ldap_authentication_conf.seamlessLogin'),
    ];

    $form['sso']['cookieExpire'] = [
      '#type' => 'select',
      '#title' => t('Cookie Lifetime'),
      '#description' => $this->t('If using the seamless login, a ' .
        'cookie is necessary to prevent automatic login after a user ' .
        'manually logs out. Select the lifetime of the cookie.'),
      '#default_value' => $config->get('ldap_authentication_conf.cookieExpire'),
      '#options' => [
        -1 => t('Session'),
        0 => t('Immediately')
      ],
    ];

    $form['sso']['ldapImplementation'] = [
      '#type' => 'select',
      '#title' => t('Authentication Mechanism'),
      '#description' => $this->t('Select the type of authentication mechanism you are using.'),
      '#default_value' => $config->get('ldap_authentication_conf.ldapImplementation'),
      '#options' => [
        'mod_auth_sspi' => t('mod_auth_sspi'),
        'mod_auth_kerb' => t('mod_auth_kerb'),
      ],
    ];

    $form['sso']['ssoExcludedPaths'] = [
      '#type' => 'textarea',
      '#title' => t('SSO Excluded Paths'),
      '#description' => $this->t('Which paths will not check for SSO? cron.php is common example. Specify pages by using their paths. Enter one path per line. The \'*\' character is a wildcard.
        Example paths are %blog for the blog page and %blog-wildcard for every personal blog. %front is the front page.',
        ['%blog' => 'blog', '%blog-wildcard' => 'blog/*', '%front' => '<front>']),
      '#default_value' => LdapAuthenticationConfiguration::arrayToLines($config->get('ldap_authentication_conf.ldapImplementation'))
    ];

    $form['sso']['ssoExcludedHosts'] = array(
      '#type' => 'textarea',
      '#title' => t('SSO Excluded Hosts'),
      '#description' => $this->t('If your site is accessible via multiple hostnames, you may only want
        the LDAP SSO module to authenticate against some of them. To exclude
        any hostnames from SSO, enter them here. Enter one host per line.'),
      '#default_value' => LdapAuthenticationConfiguration::arrayToLines($config->get('ldap_authentication_conf.ssoExcludedHosts')),
    );

    $form['submit'] = array(
      '#type' => 'submit',
      '#value' => 'Save',
    );

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    $factory = \Drupal::service('ldap.servers');
    $enabled_servers = $factory->getEnabledServers();

    foreach ($form_state->get('sid') as $sid => $discard) {
      if ($enabled_servers[$sid]->get('bind_method') == Server::$bindMethodUser || $enabled_servers[$sid]->get('bind_method') == Server::$bindMethodAnonUser) {
        $methods = array(
          Server::$bindMethodUser => $this->t('Bind with Users Credentials'),
          Server::$bindMethodAnonUser => $this->t('Anonymous Bind for search, then Bind with Users Credentials'),
        );

        $form_state->setErrorByName('ssoEnabled', $this->t(
          'Single Sign On is not valid with the server !edit (id=%sid) because that server configuration uses %bind_method.  Since the user\'s credentials are never available to this module with single sign on enabled, there is no way for the ldap module to bind to the ldap server with credentials.',
          [
            '%sid' => $sid,
            '%bind_method' => $methods[$enabled_servers[$sid]->get('bind_method')],
          ]
        ));
      }
    }

  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $values = $form_state->getValues();
    $this->config('ldap_authentication.settings')
      ->set('ldap_authentication_conf.ssoExcludedPaths', LdapAuthenticationConfiguration::linesToArray($values['ssoExcludedPaths']))
      ->set('ldap_authentication_conf.ssoExcludedHosts', LdapAuthenticationConfiguration::linesToArray($values['ssoExcludedHosts']))
      ->set('ldap_authentication_conf.ssoRemoteUserStripDomainName', $values['ssoRemoteUserStripDomainName'])
      ->set('ldap_authentication_conf.seamlessLogin', $values['seamlessLogin'])
      ->set('ldap_authentication_conf.cookieExpire', $values['cookieExpire'])
      ->set('ldap_authentication_conf.ldapImplementation', $values['ldapImplementation'])
      ->save();
  }

}
